import { extendObservable, action } from 'mobx';
import { BaseFormModel } from './FormModel';
import {copy, values, isEmpty} from '../utils';
import crfFormEdit, {field_types} from './FormEditModel';
import crfRelation from './RelationModel';
import { 
    CRFTextModel,
    CRFRaidoModel, 
    mapType
} from './FieldModel';

export const title_position = {
    TOP: 'top',
    LEFT: 'left',
    RIGHT: 'right'
}

let cell_nodes = { };

class FieldDesignModel extends BaseFormModel {
     constructor(){
        super();
        extendObservable(this, {
            formName: '', // 表单名称
            selectKey: '', // 拖动是选择的布局id
            mainElement: null, // 布局主窗口的Element
            fieldsCollection: [ ], // 问题集合
            headerStyle: {
                fontSize: '18px',
                textAlign: 'center',
                fontWeight: 400,
                color: '#333'
            },
            border:{
                color: '#F0F0F0',
                show: true 
            },
            is_static: false,
            layouts:[  // 布局集合
                this.getlayoutItem() 
            ],
            get fields(){
                return this.fieldsCollection.filter(item => item.field_type !== field_types.LABEL)
            },
            /**
             * @name 初始化
             * @param name 标题的名称
             * @param headerStyle 表头样式
             */
            init: action(({ name, headerStyle }) => {
                //this.design = [ ];
                this.formName = name;
                if( headerStyle ){
                    this.headerStyle = Object.assign({}, this.headerStyle, headerStyle)
                }
            }),
            /**
             * @name 更改标题
             * @param name 标题
             */
            changeFormName: action(name => {
                this.change = true;
                this.formName = name;
            }),
            /**
             * @name 更改标题样式
             */
            changeHeaderStyle: action(style => {
                this.change = true;
                this.headerStyle = Object.assign({}, this.headerStyle, style);
            }),
            changeBorder: action(border => {
                this.border = Object.assign({}, this.border, border);
            }),
            /**
             * @name 设置主窗口的Element
             */
            setMainElement: action(_el => {
                this.mainElement = _el;
            }),
            /**
             * @name 通过field_id获取field的store和view
             * @param id field_id
             */
            getField: id => {
                const _store = this.fieldsCollection.filter(item => item.id === id);
                return {
                    store: _store[0],  
                    view: mapType.getView( _store[0].field_type )
                };
            },
            /**
             * @name 布局更改时更新layouts
             * @param layouts:[{i,x,y,w,h}]
             */
            layoutsChange: action(layouts => {
                let _layouts = copy(this.layouts);
                for(let i = 0; i < layouts.length; i++){
                    const _index = _layouts.findIndex(item => item.key === layouts[i].i);
                    _layouts[_index].layout = {
                        x: layouts[i].x,
                        y: layouts[i].y,
                        w: layouts[i].w,
                        h: layouts[i].h
                    };
                    if( cell_nodes[layouts[i].i] ){
                        _layouts[_index].area = this.getCellPosition(cell_nodes[layouts[i].i])
                    }   
                };
                this.layouts = _layouts;
            }),
            /**
             * @name 添加一个布局元素
             */
            addCell: action(( )=> {
                let _y = 0, _h = 8;
                let _layouts = copy(this.layouts);
                for(let i = 0; i < _layouts.length; i++){
                    let { y, h } = _layouts[i].layout;
                    if(y > _y){
                        _y = y;
                        _h = h;
                    };
                };
                _layouts.push(this.getlayoutItem({
                    layout: { x: 0, y: _y + _h, w: 12, h: 6 }
                }));
                this.layouts = _layouts;
            }),
            /**
             * @name 删除一个布局元素
             * @param key 选择元素的唯一标识
             */
            deleteCell: action(key => {
                let _layouts = copy(this.layouts);
                const _index = _layouts.findIndex(item => item.key === key);
                _layouts.splice(_index, 1);
                this.layouts = _layouts;
                delete cell_nodes[key];
            }),
            /**
             * @name 储存布局元素的DOM节点
             * @param key 布局元素的唯一标识
             * @param node 布局元素的节点
             */
            setCellNode: (key, node) => {
                cell_nodes[key] = node.offsetParent;
                this.setCellPosition( key );
            },

            changeCellContentVertical: action((key, value) => {
                let _layouts = copy(this.layouts);
                let _index = _layouts.findIndex(item => item.key === key);
                _layouts[_index].cellVertical = value;
                this.layouts = _layouts;
            }),
            /**
             * @name 设置布局元素的有效区域（当有field拖动时通过该区域计算是否在次元素内）
             * @param key 布局元素的唯一标识
             */
            setCellPosition: action(key => {
                let _layouts = copy(this.layouts);
                const _index = _layouts.findIndex(item => item.key === key);
                _layouts[_index].area = this.getCellPosition(cell_nodes[key]);
                this.layouts = _layouts;
            }),
            /**
             * @name 获取布局元素节点在DOM内的坐标
             * @param node 布局元素的DOM节点
             */
            getCellPosition: action((node) => {
                const _offset = eval(node.style.transform.replace(/translate/,'').replace(/px/g, '').replace(/\(/,'[' ).replace(/\)/, ']'));
                let _pos = {
                    x: 0,
                    y: 0
                };
                if( _offset ){
                    _pos.x = _offset[0],
                    _pos.y = _offset[1]
                };
                crfFormEdit.getOffeset(_pos, node);
                const _width = node.clientWidth;
                const _Height = node.clientHeight;
                const _area = {
                    x:{
                        min: _pos.x,
                        max: _pos.x + _width
                    },
                    y:{
                        min: _pos.y,
                        max: _pos.y + _Height
                    }
                };
                return _area;
            }),
            /**
             * @name 当有field拖动时确定在哪个布局元素内
             * @param l 拖动元素的横坐标
             * @param t 拖动元素的纵坐标
             */
            changeElementInit: action((l, t) => {
                let _layouts = copy(this.layouts);
                this.selectKey = '';
                const _top = this.mainElement.scrollTop;
                const _left = this.mainElement.scrollLeft;
                let backgroundColor = 'rgb(255, 255, 255)';
                for( let i = 0; i < _layouts.length; i++ ){
                    const { x, y } = _layouts[i].area;
                    if(( l <= x.max - _left && l >=x.min - _left ) && ( t <= y.max - _top && t >= y.min - _top )){
                        this.selectKey = _layouts[i].key;
                        backgroundColor = '#F9F9F9';
                    }else{
                        backgroundColor = 'rgb(255, 255, 255)';
                    };
                    _layouts[i].style = Object.assign({}, _layouts[i].style, { backgroundColor });
                };
                this.layouts = _layouts;
            }),
            /**
             * @name 清除所有布局元素的选中状态
             */
            setElementInitCancel: action(() => {
                let _layouts = copy(this.layouts);
                for(let i = 0; i < _layouts.length; i++){
                    _layouts[i].style = Object.assign({}, _layouts[i].style, { backgroundColor: 'rgb(255,255,255)' });
                };
                this.layouts = _layouts;
            }),
            /**
             * @name 在fieldCollection添加一道问题-*
             */
            addField: action(( type ) => {
                if( this.selectKey === '' ){
                    return void 0;
                };
                const _store = mapType.getStore(type);
                _store.setFieldOption({id: (new Date()).valueOf().toString()});
                let _fields = copy(this.fieldsCollection);
                _fields.push( _store );
                this.fieldsCollection = _fields;
                let _layouts = copy(this.layouts);
                let _index = _layouts.findIndex(item => item.key === this.selectKey);
                _layouts[_index].field_id = _store.id;
                this.layouts = _layouts;
            }),
            changeFieldCollection: action((id, value) => {
                const _collection = copy( this. fieldsCollection );
                const _index = _collection.findIndex(item => item.id === id);
                _collection[_index].setFieldOption(value);
                this.fieldsCollection = _collection;
            }),
            clearFieldValue: action(() => {
                let _fields = copy(this.fieldsCollection);
                for(let i = 0; i < _fields.length; i++){
                    _fields[i].clearValue();
                };
                this.fieldsCollection = _fields;
            }),
            addRelation: action (id => {
                if( crfRelation.ctrl && !isEmpty(id) ){
                    this.is_static = true;
                    //crfRelation.changeShowCanvas();
                    console.log( id );
                };
            })
        })
    }
    /**
     * @name 获取一个布局元素的初始模板
     * @param {*} options 
     */
    getlayoutItem(options){
        return Object.assign({
            key: (new Date()).valueOf().toString(),
            field_id: '',
            cellVertical: 'top',
            style:{
                backgroundColor: 'rgba(255, 255, 255,1)'
            },
            layout:{ x: 0, y: 0, w: 12, h: 8 }
        }, options);
    }

    get fontSizeOptions(){
        let _arr = [ ];
        for( let i = 12; i < 64; i++ ){
            _arr.push( i );
        };
        return _arr
    };

    get fontWeightOptions(){
        return [
            'normal',
            '400',
            '500',
            '600',
            'bold'
        ];
    };

    fieldName( id ){
        const _field = this.fields.filter(item => item.id === id);
        if( _field.length > 0 ){
            return _field[0].label;
        };
        return '未定义';
    };

    get textAlignOptions(){
        return [{
            value: 'left',
            label: '左'
        },{
            value: 'center',
            label: '中'
        },{
            value: 'right',
            label: '右'
        }];
    };
};

const crfDesign = new FieldDesignModel();

export default crfDesign;