import { extendObservable, action, observable } from 'mobx';
import {isObject, getValueByDeepName, setByName, copy} from '../utils';
import {field_types} from './FormEditModel';
import app from '../app';
import validate from '../Validate';
import {
    TextView, 
    AutoCompleteView,
    RadioView,
} from './FieldView';

import {
    TitleView,
    PlaceHolderView,
    CodeView,
    ShowLabelView,
    RequiredView,
    DescriptionView,
    InlineView,
    OptionsInlineView,
    SetOptionsView,
    ValidateView 
} from './FieldSetting';

const edit_title = {
    BASE: {
        type: '_base',
        label: '基础信息',
        icon: 'icon-rulers'
    },
    OPTIONS: {
        type: '_options',
        label: '属性设置',
        icon: 'icon-cog6'
    },
    VAILDATE: {
        type: '_vaildate',
        label: '数据验证',
        icon: 'icon-clipboard2'
    },
    DEPEND: {
        type: '_depend',
        label: '逻辑关系',
        icon: 'icon-git-pull-request'
    }
};

const settings = {
    TITLE: TitleView,
    PLCEHOLDER: PlaceHolderView,
    CODE: CodeView,
    SHOWLABEL: ShowLabelView,
    REQUIRED: RequiredView,
    DESCRIPTION: DescriptionView,
    INLINE: InlineView,
    OPTIONSINLINE: OptionsInlineView,
    SETOPTIONS: SetOptionsView,
    VALIDATECODE: ValidateView
}

const getFieldOptions = function(options={ }){
    return Object.assign({}, {
        description: '', //备注信息
        // code_validators: [{
        //     value: '',
        //     message: ''
        // }] //逻辑验证
    }, options);
};

const getValidate = ( options=[] ) => {
    return [{
        val: validate.require,
        message: '必填项不能为空' 
    }].concat(options)
}

const getSettings = (options={}) => {
    return Object.assign({}, {
        [edit_title.BASE.type]: [
            settings.CODE,
            settings.TITLE,
            settings.PLCEHOLDER,
            settings.SHOWLABEL,
            settings.INLINE,
            settings.DESCRIPTION
        ],
        [edit_title.VAILDATE.type]: [
            settings.REQUIRED,
            settings.VALIDATECODE
        ],
        [edit_title.DEPEND.type]: []
    }, options)
}

class CRFFieldBaseModel {
    constructor(options){
        let _default = {
            id: 0,
            code: '0',
            get cid(){
                return 'c' + this.id ;
            },
            setting_selected: edit_title.BASE.type,
            setting_title: [
                edit_title.BASE,
                edit_title.VAILDATE,
                edit_title.DEPEND
            ],
            settings: getSettings(),
            label: '标题', // 问题标题
            field_options: getFieldOptions(),
            validates: [],
            value: '',
            hasError: false,
            errorMsg: '',
            disabled: false,
            showLabel: true, // 是否显示标题
            inline: false, // 标题与输入是否水平排列
            field_type: field_types.TEXT, // 为题类型
            required: false, // 是否必填
            get code_validators(){
                return this.validates.filter(item => item.type === 'error')
            },
            get code2_validators(){
                return this.validates.filter(item => item.type === 'tip')
            },
            addValidate: type => {
                let _validates = copy(this.validates);
                _validates.push({
                    value: '',
                    message: `${type} message`,
                    type
                });
                return _validates;
            },
            changeValidates: (index, name, value) => {
                let _validates = copy( this.validates );
                _validates[index][name] = value;
                return _validates;
            },
            deleteValidates: index => {
                let _validates = copy( this.validates );
                _validates.splice(index, 1);
                return _validates;
            },
            changeValue: action((value) => {
                this.value = value;
                this.hasError = false;
                for( let i = 0; i < this.validates.length; i++ ){
                    if(!this.validates[i].val(value)){
                        this.hasError = true;
                        errorMsg = this.validates[i].message;
                    };
                };
            }),
            setFieldOption: action((_field)=>{
                //console.log( _field )
                for( let key in _field ){
                    const _fieldArr = key.split('.');
                    if( _fieldArr.length > 1){
                        const _key = _fieldArr.splice(0, 1);
                        this[_key] = setByName( this[_key], _fieldArr.join('.'), _field[key] );
                        continue;
                    };
                    this[key] = _field[key];
                };
            }),
            clearValue: action(() => {
                this.value = '';
            }),
            getOptions: name => {
                return getValueByDeepName(this, name)
            }
        };
        if( isObject(options) ){
            _default = Object.assign({}, _default, options);
        };

        extendObservable(this, _default);
    };
};

export class CRFTextModel extends CRFFieldBaseModel {
    constructor(options){
        super({
            placeholder: '', // 为空提示
        });
    };
};

export class CRFAutoCompleteModel extends CRFFieldBaseModel {
    constructor(options){
        super({
            field_type: field_types.AUTOCOMPLETE,
            loading: false, // 加载loading
            searchCodes: [], // 搜索结果
            onSearchChange: action( async value => {
                try{
                    this.searchCodes = [];
                    this.loading = true;
                    const resp = await app.http.searchCode({q: value});
                    this.loading = false;
                    this.searchCodes = resp.data;
                }catch(error){
                    console.log( error );
                };
            })
        });
    };
};

export class CRFRadioModel extends CRFFieldBaseModel{
    constructor(options){
        super({
            field_type: field_types.RADIO,
            setting_title: [
                edit_title.BASE,
                edit_title.OPTIONS,
                edit_title.VAILDATE,
                edit_title.DEPEND
            ],
            settings: getSettings({
                [edit_title.OPTIONS.type]:[
                    settings.OPTIONSINLINE,
                    settings.SETOPTIONS
                ]
            }),
            field_options: getFieldOptions({
                inline: true,
                options: [{
                    code: '0',
                    label: '0= 未知',
                    id: 'default_0'
                },{
                    code: '1',
                    label: '1= 未知',
                    id: "default_1"
                }]
            }),
            changeOptionsIndex: layouts => {
                let _options = copy(this.field_options.options);
                let _new_options = new Array(_options.length);
                for( let i = 0; i < layouts.length; i++ ){
                    let item = _options.filter(item => item.id === layouts[i].i);
                    _new_options[layouts[i].y] = item[0];
                };
                return _new_options;
            },
            addOption: id => {
                let _options = copy(this.field_options.options);
                const _index = _options.findIndex(item => item.id === id);
                const _code = this.getOptionCode(_options.length - 1, _options);
                _options.splice(_index + 1, 0, {
                    code: _code,
                    label: _code + '= 未知',
                    id: (new Date()).valueOf().toString()
                });
                return _options;
            },
            deleteOption: id => {
                let _options = copy(this.field_options.options);
                const _index = _options.findIndex(item => item.id === id);
                _options.splice(_index, 1);
                return _options;
            },
            getOptionCode(code, options){
                let _code = code + 1;
                const _index =  options.findIndex(item => item.code === _code + '');
                if( _index !== -1 ){
                    return this.getOptionCode(_code, options)
                }else{
                    return _code + ''
                }
            }
        });
    };
};

class MapType {
    getStore(type){
        switch( type ){
            case field_types.TEXT:
                return new CRFTextModel();
            case field_types.AUTOCOMPLETE:
                return new CRFAutoCompleteModel();
            case field_types.RADIO:
                return new CRFRadioModel();
            default:
                return new CRFTextModel();
        };
    };

    getView(type){
        switch( type ){
            case field_types.TEXT:
                return TextView;
            case field_types.AUTOCOMPLETE:
                return AutoCompleteView;
            case field_types.RADIO:
                return RadioView;
            default :
                return TextView;
        };
    };
};

export const mapType = new MapType()